      
c	***************************************************************
c	Program that solves for the equilibrium of a CES or a Cobb-Douglas 
c	dynamic Heckscher Ohlin model (as described in Bajona and Kehoe (2010)
c	NOTE: the program assumes:
c	      - Good 1 is the capital intensive good
c	      - Complete depreciation
c	***************************************************************    

	implicit real*8 (a-h,o-z)
	common nt,nn,beta,b,alpha1,alpha2,theta1,theta2,a1,a2,d,xl,
     1  aa,dd,xkss,xk,ratx	 
	dimension xk(2,50),xl(2),ratx(50),p(2,50),c(2,50),w(2,50),r(2,50)
     1  ,y(2,2,50),xkk(2,2,50),xll(2,2,50),cc(2,2,50),xx(2,2,50),
     2  zkap1(50),zkap2(50)
c
c     set number of periods
	nt=15
c
c     set number of variables and unknowns
      nn=3*nt-1
c
c     set parameters
	beta=0.95d0
	b=-0.5d0
	alpha1=0.8d0
	alpha2=0.2d0
	theta1=10.d0
	theta2=10.d0
	a1=0.5d0
	a2=1.d0-a1
	d=1.d0
	xl(1)=10.d0
	xl(2)=10.d0
      	xxl=xl(1)+xl(2)
	zk10=0.55d0
	zk20=0.15d0
	xk10=zk10*xl(1)
	xk20=zk20*xl(2)
c
c	calculate initial aggregate capital-labor ratio
	zk0=(xl(1)*zk10+xl(2)*zk20)/(xl(1)+xl(2))
c
c     calculate parameters of one-sector production function 
      if(dabs(b).lt.1.d-8)goto 10
      aa11=(a1*alpha1*theta1**b)**(1.d0/(1.d0-b))
	aa21=(a2*alpha2*theta2**b)**(1.d0/(1.d0-b))
	aa12=(a1*(1.d0-alpha1)*theta1**b)**(1.d0/(1.d0-b))
	aa22=(a2*(1.d0-alpha2)*theta2**b)**(1.d0/(1.d0-b))
	aa=(aa11+aa21)**(1.d0-b)/
	1  ((aa11+aa21)**(1.d0-b)+(aa12+aa22)**(1.d0-b))												   
	dd=d*((aa11+aa21)**(1.d0-b)+(aa12+aa22)**(1.d0-b))**(1.d0/b)
	goto 20
 10   continue
      aa=a1*alpha1+a2*alpha2
	dd=d*(theta1*a1*alpha1**alpha1*(1.d0-alpha1)**(1.d0-alpha1))**a1
	1  *(theta2*a2*alpha2**alpha2*(1.d0-alpha2)**(1.d0-alpha2))**a2
	2  /(aa**aa*(1.d0-aa)**(1.d0-aa))
 20   continue
c
c     calculate steady state aggregate capital-labor ratio
      if(dabs(b).ge.1.d-8)xkss=((1.d0-aa)*xxl**b/((aa*beta*dd)
	1  **(-b/(1.d0-b))-aa))**(1.d0/b)
      if(dabs(b).lt.1.d-8)xkss=(aa*beta*dd)**(1.d0/(1.d0-aa))*xxl
	zkss=xkss/xxl
c
c     calculate kappas for cone of divesification
	if(dabs(b).lt.1.d-8)goto 30
	zkapp1=(alpha1/(1.d0-alpha1))**(1.d0/(1.d0-b))*(aa12+aa22)
	1  /(aa11+aa21)
	zkapp2=(alpha2/(1.d0-alpha2))**(1.d0/(1.d0-b))*(aa12+aa22)
	1  /(aa11+aa21)
	goto 40
 30   continue
      zkapp1=(alpha1/(1.d0-alpha1))*(1-aa)/aa
	zkapp2=(alpha2/(1.d0-alpha2))*(1-aa)/aa
 40   continue
c
c     calculate steady state x2/x1
      zl1=(1.d0-zkapp2)/(zkapp1-zkapp2)
	xl1ss=zl1*xxl
	xk1ss=zkapp1*xl1ss*zkss
	zl2=(zkapp1-1.d0)/(zkapp1-zkapp2)
 	xl2ss=zl2*xxl
	xk2ss=zkapp2*xl2ss*zkss
     	x1ss=phi1(xk1ss,xl1ss)
	x2ss=phi2(xk2ss,xl2ss)
	ratxss=x2ss/x1ss
c
c     guess inital x2/x1
    	xkk10=zkapp1*xl1ss*zk0
    	xkk20=zkapp2*xl2ss*zk0 
     	x10=phi1(xkk10,xl1ss)
	x20=phi2(xkk20,xl2ss)
	ratx0=x20/x10
c
c     guess income shares
	if(dabs(b).ge.1.d-8)yy1=dd*(aa*xk10**b+(1.d0-aa)*xl(1)**b)
	1  **(1.d0/b)
	if(dabs(b).ge.1.d-8)yy2=dd*(aa*xk20**b+(1.d0-aa)*xl(2)**b)
	1  **(1.d0/b)
	if(dabs(b).lt.1.d-8)yy1=dd*xk10**aa*xl(1)**(1.d0-aa)
	if(dabs(b).lt.1.d-8)yy2=dd*xk20**aa*xl(2)**(1.d0-aa)
	yy1=yy1/(yy1+yy2)
	yy2=1.d0-yy1
c
c     guess path
      xk(1,1)=xk10
	xk(2,1)=xk20	
	ratx(1)=ratx0
      xk1ss=yy1*xkss
	xk2ss=yy2*xkss
      do 50 it=2,5
	  z=(it-1.d0)/5.d0
	  xk(1,it)=(1.d0-z)*xk10+z*xk1ss
	  xk(2,it)=(1.d0-z)*xk20+z*xk2ss
	  ratx(it)=(1.d0-z)*ratx0+z*ratxss
 50   continue
      do 60 it=6,nt+1
	  xk(1,it)=xk1ss
	  xk(2,it)=xk2ss
	  ratx(it)=ratxss
 60   continue
c
c	solve for equilibrium using Newton's method
	call solve
c
	if(dabs(b).ge.1.d-8)sigma=1.d0/(1.d0-b)
	if(dabs(b).ge.1.d-8)bsigma=b/(1.d0-b)
	do 180 it=1,nt+1
	  if(dabs(b).lt.1.d-8)goto 120
	  p(1,it)=a1*d*(a1+a2*ratx(it)**b)**(1.d0/b-1.d0)
	  p(2,it)=a2*ratx(it)**(b-1.d0)*d*(a1+a2*ratx(it)**b)
     1    **(1.d0/b-1.d0)
	  zkap1(it)=(alpha1/(1.d0-alpha1))**sigma*(((1.d0-alpha2)**sigma
	1    *(theta2*p(2,it))**bsigma-(1.d0-alpha1)**sigma
     2    *(theta1*p(1,it))**bsigma)/(alpha1**sigma
     3    *(theta1*p(1,it))**bsigma-alpha2**sigma
     4    *(theta2*p(2,it))**bsigma))**(1.d0/b)
	  zkap2(it)=((alpha2*(1.d0-alpha1))/(alpha1*(1.d0-alpha2)))**sigma
     1    *zkap1(it)
	  goto 130
 120    continue
	  p(1,it)=a1*d*ratx(it)**a2
	  p(2,it)=a2*d*ratx(it)**(a2-1.d0)
        zkap1(it)=((theta2*p(2,it)*alpha2**alpha2
     1    *(1.d0-alpha2)**(1.d0-alpha2))/(theta1*p(1,it)*alpha1**alpha2
	2    *(1.d0-alpha1)**(1.d0-alpha2)))**(1.d0/(alpha1-alpha2))
	  zkap2(it)=((alpha2*(1.d0-alpha1))/(alpha1*(1.d0-alpha2)))
	1    *zkap1(it)
 130    continue
        do 170 i=1,2
	    if((xk(i,it)/xl(i)).lt.zkap2(it))goto 140
	    if((xk(i,it)/xl(i)).gt.zkap1(it))goto 150
	    zl1=(xk(i,it)/xl(i)-zkap2(it))/(zkap1(it)-zkap2(it))
	    xll(i,1,it)=zl1*xl(i)
	    xkk(i,1,it)=zkap1(it)*xll(i,1,it)
          zl2=(zkap1(it)-xk(i,it)/xl(i))/(zkap1(it)-zkap2(it))
 	    xll(i,2,it)=zl2*xl(i)
	    xkk(i,2,it)=zkap2(it)*xll(i,2,it)
	         yz=dd*(aa*xk(i,it)**b+(1.d0-aa)*xl(i)**b)**(1.d0/b)
	    yz1=f(phi1(xkk(i,1,it),xll(i,1,it)),
     1        phi2(xkk(i,2,it),xll(i,2,it)))
	    if(dabs(b).ge.1.d-8)r(i,it)=p(1,it)*alpha1
	1      *xkk(i,1,it)**(b-1.d0)*theta1*(alpha1*xkk(i,1,it)**b
     2      +(1.d0-alpha1)*xll(i,1,it)**b)**(1.d0/b-1.d0)
	    if(dabs(b).ge.1.d-8)w(i,it)=p(1,it)*(1.d0-alpha1)
	1      *xll(i,1,it)**(b-1.d0)*theta1*(alpha1*xkk(i,1,it)**b+
     2      (1.d0-alpha1)*xll(i,1,it)**b)**(1.d0/b-1.d0)
		if(dabs(b).lt.1.d-8)r(i,it)=p(1,it)*alpha1*theta1*
     1      xkk(i,1,it)**(alpha1-1.d0)*xll(i,1,it)**(1.d0-alpha1)
		if(dabs(b).lt.1.d-8)w(i,it)=p(1,it)*(1.d0-alpha1)*theta1*
     1      xkk(i,1,it)**alpha1*xll(i,1,it)**(-alpha1)
          goto 160
 140      continue
          xll(i,1,it)=0.d0
	    xll(i,2,it)=xl(i)
	    xkk(i,1,it)=0.d0
	    xkk(i,2,it)=xk(i,it)
		if(dabs(b).ge.1.d-8)r(i,it)=p(2,it)*alpha2
     1      *xkk(i,2,it)**(b-1.d0)*theta2*(alpha2*xkk(i,2,it)**b
     2      +(1.d0-alpha2)*xll(i,2,it)**b)**(1.d0/b-1.d0)
	    if(dabs(b).ge.1.d-8)w(i,it)=p(2,it)*(1.d0-alpha2)
	1      *xll(i,2,it)**(b-1.d0)*theta2*(alpha2*xkk(i,2,it)**b+
     2      (1.d0-alpha2)*xll(i,2,it)**b)**(1.d0/b-1.d0)
		if(dabs(b).lt.1.d-8)r(i,it)=p(2,it)*alpha2*theta2*
     1      xkk(i,2,it)**(alpha2-1.d0)*xll(i,2,it)**(1.d0-alpha2)
		if(dabs(b).lt.1.d-8)w(i,it)=p(2,it)*(1.d0-alpha2)*theta2*
     1      xkk(i,2,it)**alpha2*xll(i,2,it)**(-alpha2)  
	    goto 160
 150      continue
          xll(i,1,it)=xl(i)
	    xll(i,2,it)=0.d0
	    xkk(i,1,it)=xk(i,it)
	    xkk(i,2,it)=0.d0
 	    if(dabs(b).ge.1.d-8)r(i,it)=p(1,it)*alpha1
     1      *xkk(i,1,it)**(b-1.d0)*theta1*(alpha1*xkk(i,1,it)**b+
     2      (1.d0-alpha1)*xll(i,1,it)**b)**(1.d0/b-1.d0)
	    if(dabs(b).ge.1.d-8)w(i,it)=p(1,it)*(1.d0-alpha1)
	1      *xll(i,1,it)**(b-1.d0)*theta1*(alpha1*xkk(i,1,it)**b+
     2      (1.d0-alpha1)*xll(i,1,it)**b)**(1.d0/b-1.d0)
		if(dabs(b).lt.1.d-8)r(i,it)=p(1,it)*alpha1*theta1*
     1      xkk(i,1,it)**(alpha1-1.d0)*xll(i,1,it)**(1.d0-alpha1)
		if(dabs(b).lt.1.d-8)w(i,it)=p(1,it)*(1.d0-alpha1)*theta1*
     1      xkk(i,1,it)**alpha1*xll(i,1,it)**(-alpha1)
 160      continue
c
		y(i,1,it)=phi1(xkk(i,1,it),xll(i,1,it))
	    y(i,2,it)=phi2(xkk(i,2,it),xll(i,2,it))
		xx(i,1,it)=xk(i,it+1)/f(1.d0,ratx(it))
	    xx(i,2,it)=xk(i,it+1)/f(1.d0/ratx(it),1.d0)
	    xinci=p(1,it)*y(i,1,it)+p(2,it)*y(i,2,it)
	    c(i,it)=xinci-xk(i,it+1)
		cc(i,1,it)=c(i,it)/f(1.d0,ratx(it))
	    cc(i,2,it)=c(i,it)/f(1.d0/ratx(it),1.d0)
 170    continue
 180  continue
c
      open(unit=9,file='DynHOResults.txt')
 1111 format(1x,'        k1          k2          kap1        kap2          
     1 kk1         kk2')
      write(6,1111)												  
	write(9,1111)
      do 190 it=1,nt
	  zk=(xk(1,it)+xk(2,it))/(xl(1)+xl(2))
	  write(6,1112)it-1,xk(1,it)/xl(1),xk(2,it)/xl(2),zkapp1*zk,
     1    zkapp2*zk,zkap1(it),zkap2(it)
	  write(9,1112)it-1,xk(1,it)/xl(1),xk(2,it)/xl(2),zkapp1*zk,
     1    zkapp2*zk,zkap1(it),zkap2(it)
 1112   format(1x,i3,6f12.6)
  190 continue
c
      write(6,*)' '
      open(unit=11,file='DynHOMore.txt')
 1113 format(1x,'     rinc1     rinc2     y11       y21       y12    
     1y22       r1        r2')  
      write(6,1113)												  
	write(11,1113)
      do 200 it=1,nt
	  xinc1=p(1,it)*y(1,1,it)+p(2,it)*y(1,2,it)
	  xinc2=p(1,it)*y(2,1,it)+p(2,it)*y(2,2,it)
	  rinc1=2.d0*xinc1/(xinc1+xinc2)-1.d0
	  rinc2=2.d0*xinc2/(xinc1+xinc2)-1.d0
	  write(6,1114)it-1,rinc1,rinc2,y(1,1,it),y(1,2,it),y(2,1,it),
	1    y(2,2,it),r(1,it),r(2,it)
	  write(11,1114)it-1,rinc1,rinc2,y(1,1,it),y(1,2,it),y(2,1,it),
	1    y(2,2,it),r(1,it),r(2,it)
 1114   format(1x,i2,f7.4,7f10.4)
  200 continue
c
      stop
	end